LBL<-function(dat, numSNPs, maxMissingGenos = 1, allelic = TRUE, haplo.baseline = "missing", cov.baseline = "missing", complex.sampling = FALSE, n.stra = NULL, interaction.stra = TRUE, interaction.env = TRUE, interaction.model = "i", names.dep = "missing", a = 20, b = 20, start.beta = 0.01, gamma = 0.01, lambda = 1, D = 0, e = 0.1, seed = NULL, burn.in = NULL, num.it = NULL)
{

    haplos.list<-pre.hapassoc(dat, numSNPs=numSNPs, pooling.tol=0, allelic=allelic, maxMissingGenos=maxMissingGenos)
    var.baseline = cov.baseline
    interaction.cov = interaction.env

    ####################################################################################################
    #Non-complex sampling methods
    ####################################################################################################
    if(complex.sampling==FALSE)
    {

        n.cov=dim(haplos.list$nonHaploDM)[2]-1#num of covariates
        cov.baseline=var.baseline

        ##################################################
        #LBL
        ##################################################
        if(n.cov==0){
    
        temp=cbind(haplos.list$ID, haplos.list$wt, haplos.list$nonHaploDM, haplos.list$haploDM)
        temp=temp[order(temp[,1]),]
        haplos.list$ID=temp[,1]
        haplos.list$wt=temp[,2]
        haplos.list$nonHaploDM=temp[,3]
        haplos.list$haploDM=temp[4:ncol(temp)]
    
        if (is.null(seed)==FALSE) set.seed(seed)
        if (is.null(burn.in)==TRUE) burn.in=20000
        if (is.null(num.it)==TRUE) num.it=50000
        haplo.names <- names(haplos.list$initFreq)
        freq <- haplos.list$initFreq
        freq = freq[names(haplos.list$haploDM)]
        if (haplo.baseline=="missing") haplo.baseline <- haplo.names[which.max(freq)]
        column.subset <- colnames(haplos.list$haploDM) != haplo.baseline  
        hdat <- cbind(haplos.list$nonHaploDM, haplos.list$haploDM[, column.subset])
        colnames(hdat) <- c("affected", colnames(haplos.list$haploDM[, column.subset]))
        ID <- haplos.list$ID
        N <- sum(haplos.list$wt)
        y <- as.numeric(hdat[, 1])# y <- as.numeric(hdat[colnames(hdat)!="pheno"])
        x <- data.matrix(hdat[, -1])# x <- data.matrix(hdat[colnames(hdat)!="pheno"])
        colnames(x)<-NULL
    
        freq.new<-freq[names(haplos.list$haploDM[, column.subset])]
        freq.new[length(freq.new)+1]<-freq[haplo.baseline]
        freq.new<-as.vector(freq.new)
        num.haplo.id<-as.vector(table(ID))
        x.length<-as.integer(dim(x)[2])
        unique.x<-unique(x)
    
        beta=rep(start.beta, x.length)
        beta.out<-numeric((num.it-burn.in)*x.length)
        lambda.out<-numeric(num.it-burn.in)
        freq.out<-numeric((num.it-burn.in)*(x.length+1))
        D.out<-numeric(num.it-burn.in)
        
        out<-.C("mcmc_without_cov", as.integer(x), as.integer(dim(x)[1]), as.integer(y), as.integer(N), as.integer(num.haplo.id), as.integer(x.length), as.double(freq.new), as.double(D), as.double(beta), as.double(a), as.double(b), as.integer(t(unique.x)), as.integer(dim(unique.x)[1]), as.double(lambda), as.integer(num.it), as.integer(burn.in), beta.out=as.double(beta.out), lambda.out=as.double(lambda.out), freq.out=as.double(freq.out), D.out=as.double(D.out))
    
        beta.out<-matrix(out$beta.out,nrow=num.it-burn.in, byrow=TRUE)
        freq.out<-matrix(out$freq.out,nrow=num.it-burn.in, byrow=TRUE)
    
        post.mean.beta<-numeric(x.length)
        ci.beta<-numeric(x.length*2)
        post.mean.freq<-numeric(x.length+1) 
        ci.freq<-numeric((x.length+1)*2)
        ci.lambda<-numeric(2)
        ci.D<-numeric(2)
    
        k<-1
        for (i in 1:x.length)
        {
            ci.beta[k]<-quantile(beta.out[,i], probs=0.025)
            ci.beta[k+1]<-quantile(beta.out[,i], probs=0.975)
            k<-k+2
            post.mean.beta[i]<- mean(beta.out[,i])
        }
        ci.beta<-matrix(ci.beta,nrow=x.length, ncol=2, byrow=TRUE)
        ci.OR<-data.frame(exp(ci.beta))
        OR<-exp(post.mean.beta)
    
        k<-1
        for (i in 1:(x.length+1))
        {
            ci.freq[k]<-quantile(freq.out[,i], probs=0.025)
            ci.freq[k+1]<-quantile(freq.out[,i], probs=0.975)
            k<-k+2
            post.mean.freq[i]<- mean(freq.out[,i])
        } 
        ci.freq<-matrix(ci.freq,nrow=(x.length+1), ncol=2, byrow=TRUE)
        
        ci.lambda<-c(quantile(out$lambda.out, probs=0.025),quantile(out$lambda.out, probs=0.975))
        ci.D<-c(quantile(out$D.out, probs=0.025),quantile(out$D.out, probs=0.975))
    
        prob.alt<-numeric(x.length)
        BF<-numeric(x.length)
        
        for (i in 1:x.length)
        {
            prob.alt[i]<-length(beta.out[,i][abs(beta.out[,i]) > e])/(num.it-burn.in)
        }
    
        for (i in 1:x.length)
        {
            prior.prob<-(b/(e+b))^a
            prior.odds<-prior.prob/(1-prior.prob)
            if (prob.alt[i]<=(100*prior.odds)/(100*prior.odds+1)) 
            {
                BF[i]<-round((prob.alt[i]/(1-prob.alt[i]))/prior.odds,4)
            } else
            BF[i]<-">100"
        }

        OR<-round(OR,4)
        ci.OR<-round(ci.OR,4)
        post.mean.freq<-round(post.mean.freq,4)
        ci.freq<-round(ci.freq,4)
        ci.lambda<-round(ci.lambda,4)
        ci.D<-round(ci.D,4)    
    
        names(BF)<-colnames(hdat[, -1])
        names(OR)<-colnames(hdat[, -1])

        ci.OR<-data.frame(colnames(hdat[, -1]),ci.OR)
        colnames(ci.OR)<-c("Hap", "Lower", "Upper")
    
        ci.freq<-data.frame(c(colnames(hdat[, -1]),haplo.baseline), ci.freq)
        colnames(ci.freq)<-c("Hap", "Lower", "Upper")
        names(post.mean.freq)<-c(colnames(hdat[, -1]), haplo.baseline)
    
        names(ci.lambda)<-c("Lower", "Upper")
        names(ci.D)<-c("Lower", "Upper")
    
        ans <- list(BF=BF, OR=OR, CI.OR=ci.OR, freq=post.mean.freq, CI.freq=ci.freq, CI.lambda=ci.lambda, CI.D=ci.D)
        }#end here
    
        ##################################################
        #LBL-GXE-I
        ##################################################
        if(n.cov>=1&&interaction.model=="i"){
    
        temp=cbind(haplos.list$ID, haplos.list$wt, haplos.list$nonHaploDM, haplos.list$haploDM)
        temp=temp[order(temp[,1]),]
        haplos.list$ID=temp[,1]
        haplos.list$wt=temp[,2]
        haplos.list$nonHaploDM=temp[,3:(3+n.cov)]
        haplos.list$haploDM=temp[(3+n.cov+1):ncol(temp)]
    
        if (is.null(seed)==FALSE) set.seed(seed)
        if (is.null(burn.in)==TRUE) burn.in=20000
        if (is.null(num.it)==TRUE) num.it=50000
        freq <- haplos.list$initFreq
        freq = freq[names(haplos.list$haploDM)]
        freq.names <- names(freq)
        if (haplo.baseline=="missing") haplo.baseline <- freq.names[which.max(freq)]
        column.subset <- colnames(haplos.list$haploDM) != haplo.baseline
        haplo.mat=haplos.list$haploDM[,column.subset]
        haplo.names=colnames(haplo.mat)
    
        freq.new<-freq[column.subset]
        freq.new[length(freq.new)+1]<-freq[haplo.baseline]
    
        if (cov.baseline=="missing") cov.baseline = rep(0, n.cov)
        haplos.list$nonHaploDM=as.matrix(haplos.list$nonHaploDM)   
        for(i in 1:n.cov)
        {
            cov.temp=dummy(haplos.list$nonHaploDM[,i+1])#transform the ith covariate into dummy variables
            cov.levels=sort(unique(haplos.list$nonHaploDM[,i+1]))
            cov.subset=(cov.levels!=cov.baseline[i])
            cov.temp=cov.temp[,cov.subset]#remove the baseline from dummy variables
            colnames.cov.temp=paste(colnames(haplos.list$nonHaploDM)[i+1], cov.levels[cov.subset], sep="")
            if(i==1)
            {
                cov.mat=cov.temp
                cov.names=colnames.cov.temp
            }else{
                cov.mat=cbind(cov.mat, cov.temp)#bind all the covariates into one matrix
                cov.names=c(cov.names, colnames.cov.temp)#bind all the covariate names into one matrix
            }
        }
    
        y <- haplos.list$nonHaploDM[,1]
        N <- sum(haplos.list$wt)
        n=dim(haplo.mat)[1]
        num.haplo.id=as.vector(table(haplos.list$ID))
    
        h.length=ncol(haplo.mat)+1
        if(interaction.cov==T){
            if (is.matrix(cov.mat)==T){x.length=ncol(haplo.mat)+ncol(cov.mat)+ncol(haplo.mat)*ncol(cov.mat)
            }else{x.length=ncol(haplo.mat)+1+ncol(haplo.mat)}
        }else{
            if (is.matrix(cov.mat)==T){x.length=ncol(haplo.mat)+ncol(cov.mat)
            }else{x.length=ncol(haplo.mat)+1}
        }
    
        haplo.map<-matrix(rep(NA,2*(dim(haplo.mat)[1])),ncol=2)
        for(i in 1:dim(haplo.mat)[1])
        {
            for(j in 1:dim(haplo.mat)[2])
            {
                if(haplo.mat[i,j]==2)
                {
                    haplo.map[i,1]=j
                    haplo.map[i,2]=j
                    break
                }
                if(haplo.mat[i,j]==1)
                {
                    if(is.na(haplo.map[i,1])==T) haplo.map[i,1]=j
                    else
                    {
                        haplo.map[i,2]=j
                        break
                    }
                }
            }
            if(is.na(haplo.map[i,1])==T)
            {
                haplo.map[i,1]=dim(haplo.mat)[2]+1
                haplo.map[i,2]=dim(haplo.mat)[2]+1
            }
            if(is.na(haplo.map[i,2])==T) haplo.map[i,2]=dim(haplo.mat)[2]+1
        }  
    
        uniq.mat<-unique(haplo.mat)
        uniq.map<-matrix(rep(NA,2*(dim(uniq.mat)[1])),ncol=2)
        for(i in 1:dim(uniq.mat)[1])
        {
            for(j in 1:dim(uniq.mat)[2])
            {
                if(uniq.mat[i,j]==2)
                {
                    uniq.map[i,1]=j
                    uniq.map[i,2]=j
                    break
                }
                if(uniq.mat[i,j]==1)
                {
                    if(is.na(uniq.map[i,1])==T) uniq.map[i,1]=j
                    else
                    {
                        uniq.map[i,2]=j
                        break
                    }
                }
            }
            if(is.na(uniq.map[i,1])==T)
            {
                uniq.map[i,1]=dim(uniq.mat)[2]+1
                uniq.map[i,2]=dim(uniq.mat)[2]+1
            }
            if(is.na(uniq.map[i,2])==T) uniq.map[i,2]=dim(uniq.mat)[2]+1
        }

        if (is.matrix(cov.mat)==T){cov.vec=apply(cov.mat, 1, function(x) paste(x, collapse=""))#concatenate each row of cov.mat
        }else{cov.vec=cov.mat}
        cov.vec.new=unique(cbind(haplos.list$ID, cov.vec))[,-1]
        y.new=unique(cbind(haplos.list$ID, y))[,-1]
        num.E=as.vector(table(cov.vec.new))#distribution of E's
    
        value.E=unique(cov.mat)#the dimention of value.E is len.E X len.dummy
        if (is.matrix(value.E)==T){
            value.E=value.E[order(unique(cov.vec)),]#reorder value.E to be consistant with num.E
            len.E=nrow(value.E)#num of different E's
            len.dummy=ncol(value.E)#num of dummy variables
        }else{
            value.E=value.E[order(unique(cov.vec))]#reorder value.E to be consistant with num.E
            len.E=length(value.E)#num of different E's
            len.dummy=1#num of dummy variables
        }

        if (is.matrix(value.E)==T){value.E.vec=apply(value.E, 1, function(x) paste(x, collapse=""))#concatenate each row of value.E
        }else{value.E.vec=value.E}
        index.E=match(cov.vec.new, value.E.vec)
    
        beta=rep(start.beta, x.length+1)
        beta.out<-numeric((num.it-burn.in)*(x.length+1))
        lambda.out<-numeric(num.it-burn.in)
        freq.out<-numeric((num.it-burn.in)*(h.length))
        D.out<-numeric(num.it-burn.in)
    
        if(interaction.cov==F)no.interaction=1
        else no.interaction=0

        out<-.C("mcmc_indep", as.integer(y.new), as.integer(N), as.integer(n), as.integer(num.haplo.id), as.integer(x.length), as.integer(h.length), as.integer(haplo.map), as.integer(dim(uniq.mat)[1]), as.integer(t(uniq.map)), as.integer(index.E), as.integer(t(value.E)), as.integer(num.E), as.integer(len.E), as.integer(len.dummy), as.double(beta), as.double(lambda), as.double(freq.new), as.double(D), as.double(a), as.double(b), beta.out=as.double(beta.out), lambda.out=as.double(lambda.out), freq.out=as.double(freq.out), D.out=as.double(D.out), as.integer(num.it), as.integer(burn.in), as.integer(no.interaction))
  
        beta.out<-matrix(out$beta.out,nrow=num.it-burn.in, byrow=TRUE)
        freq.out<-matrix(out$freq.out,nrow=num.it-burn.in, byrow=TRUE)
    
        post.mean.beta<-numeric(x.length+1)
        ci.beta<-numeric((x.length+1)*2)
        post.mean.freq<-numeric(h.length)
        ci.freq<-numeric((h.length)*2)
        ci.lambda<-numeric(2)
        ci.D<-numeric(2)
     
        k<-1
        for (i in 1:(x.length+1))
        {
            ci.beta[k]<-quantile(beta.out[,i], probs=0.025)
            ci.beta[k+1]<-quantile(beta.out[,i], probs=0.975)
            k<-k+2
            post.mean.beta[i]<-mean(beta.out[,i])
        }
        ci.beta<-matrix(ci.beta,nrow=x.length+1, ncol=2, byrow=TRUE)
        ci.OR<-data.frame(exp(ci.beta))
        OR<-exp(post.mean.beta)
        
        k<-1
        for (i in 1:(h.length))
        {
            ci.freq[k]<-quantile(freq.out[,i], probs=0.025)
            ci.freq[k+1]<-quantile(freq.out[,i], probs=0.975)
            k<-k+2
            post.mean.freq[i]<- mean(freq.out[,i])
        }
    
        ci.lambda<-c(quantile(out$lambda.out, probs=0.025),quantile(out$lambda.out, probs=0.975))
        ci.D<-c(quantile(out$D.out, probs=0.025),quantile(out$D.out, probs=0.975))
    
        prob.alt<-numeric(x.length+1)
        BF<-numeric(x.length+1)
        
        for (i in 1:(x.length+1))
        {
            prob.alt[i]<-length(beta.out[,i][abs(beta.out[,i]) > e])/(num.it-burn.in)
        }
    
        for (i in 1:(x.length+1))
        {
            prior.prob<-(b/(e+b))^a
            prior.odds<-prior.prob/(1-prior.prob)
            if (prob.alt[i]<=(100*prior.odds)/(100*prior.odds+1)) 
            {
                BF[i]<-round((prob.alt[i]/(1-prob.alt[i]))/prior.odds,4)
            } else
            BF[i]<-">100"
        }
    
        OR<-round(OR,4)
        ci.OR<-round(ci.OR,4)
        post.mean.freq<-round(post.mean.freq,4)
        ci.freq<-round(ci.freq,4)
        ci.lambda<-round(ci.lambda,4)
        ci.D<-round(ci.D,4)    
    
        if(interaction.cov==T){
            for(i in 1:len.dummy)
            {
                name.paste.temp<-paste(cov.names[i],haplo.names,sep=".")
                if(i==1){name.paste=name.paste.temp
                }else{name.paste=c(name.paste,name.paste.temp)}
            }
            pname<-c("beta0",haplo.names,name.paste,cov.names)
        }else{
            pname<-c("beta0",haplo.names,cov.names)
        }
        names(BF)<-pname
        names(OR)<-pname
    
        ci.OR<-data.frame(pname,ci.OR)
        colnames(ci.OR)<-c("Name","Lower","Upper")
    
        ci.freq<-matrix(ci.freq,nrow=(h.length),ncol=2,byrow=TRUE)
        ci.freq<-data.frame(c(haplo.names,haplo.baseline),ci.freq)
        colnames(ci.freq)<-c("Name","Lower","Upper")
        names(post.mean.freq)<-c(haplo.names,haplo.baseline)
    
        names(ci.lambda)<-c("Lower","Upper")
        names(ci.D)<-c("Lower","Upper")
    
        ans <- list(BF=BF, OR=OR, CI.OR=ci.OR, freq=post.mean.freq, CI.freq=ci.freq, CI.lambda=ci.lambda, CI.D=ci.D)
        }#ends here
    
        ##################################################
        #LBL-GXE-D
        ##################################################
        if(n.cov>=1&&interaction.model=="d"){
    
        names.cov=colnames(haplos.list$nonHaploDM)[2:(1+n.cov)]
        if (names.dep=="missing") names.dep=names.cov
        temp=cbind(haplos.list$ID, haplos.list$wt, haplos.list$nonHaploDM, haplos.list$haploDM)
        temp=temp[order(temp[,1]),]
        haplos.list$ID=temp[,1]
        haplos.list$wt=temp[,2]
        haplos.list$nonHaploDM=temp[,3:(3+n.cov)]
        haplos.list$haploDM=temp[(3+n.cov+1):ncol(temp)]
    
        if (is.null(seed)==FALSE) set.seed(seed)
        if (is.null(burn.in)==TRUE) burn.in=20000
        if (is.null(num.it)==TRUE) num.it=70000
        freq <- haplos.list$initFreq
        freq = freq[names(haplos.list$haploDM)]
        freq.names <- names(freq)
        if (haplo.baseline=="missing") haplo.baseline <- freq.names[which.max(freq)]
        column.subset <- colnames(haplos.list$haploDM) != haplo.baseline
        haplo.mat=haplos.list$haploDM[,column.subset]
        haplo.names=colnames(haplo.mat)
    
        freq.new<-freq[column.subset]
        freq.new[length(freq.new)+1]<-freq[haplo.baseline]
       
        if (cov.baseline=="missing") cov.baseline = rep(0, n.cov)
        haplos.list$nonHaploDM=as.matrix(haplos.list$nonHaploDM)
        num.dummies.each.cov=rep(0, n.cov)
        for(i in 1:n.cov)
        {
            cov.temp=dummy(haplos.list$nonHaploDM[,i+1])
            cov.levels=sort(unique(haplos.list$nonHaploDM[,i+1]))
            cov.subset=(cov.levels!=cov.baseline[i])
            cov.temp=cov.temp[,cov.subset]
            colnames.cov.temp=paste(colnames(haplos.list$nonHaploDM)[i+1], cov.levels[cov.subset], sep="")
            num.dummies.each.cov[i]=length(cov.levels)-1
            if(i==1)
            {
                cov.mat=cov.temp
                cov.names=colnames.cov.temp
            }else{
                cov.mat=cbind(cov.mat, cov.temp)
                cov.names=c(cov.names, colnames.cov.temp)
            }
        }
    
        y <- haplos.list$nonHaploDM[,1]
        N <- sum(haplos.list$wt)
        n=dim(haplo.mat)[1]
        num.haplo.id=as.vector(table(haplos.list$ID))
    
        h.length=ncol(haplo.mat)+1
        if (is.matrix(cov.mat)==T){x.length=ncol(haplo.mat)+ncol(cov.mat)+ncol(haplo.mat)*ncol(cov.mat)
        }else{x.length=ncol(haplo.mat)+1+ncol(haplo.mat)}
    
        haplo.map<-matrix(rep(NA,2*(dim(haplo.mat)[1])),ncol=2)
        for(i in 1:dim(haplo.mat)[1])
        {
            for(j in 1:dim(haplo.mat)[2])
            {
                if(haplo.mat[i,j]==2)
                {
                    haplo.map[i,1]=j
                    haplo.map[i,2]=j
                    break
                }
                if(haplo.mat[i,j]==1)
                {
                    if(is.na(haplo.map[i,1])==T) haplo.map[i,1]=j
                    else
                    {
                        haplo.map[i,2]=j
                        break
                    }
                }
            }
            if(is.na(haplo.map[i,1])==T)
            {
                haplo.map[i,1]=dim(haplo.mat)[2]+1
                haplo.map[i,2]=dim(haplo.mat)[2]+1
            }
            if(is.na(haplo.map[i,2])==T) haplo.map[i,2]=dim(haplo.mat)[2]+1
        }  
    
        uniq.mat<-unique(haplo.mat)
        uniq.map<-matrix(rep(NA,2*(dim(uniq.mat)[1])),ncol=2)
        for(i in 1:dim(uniq.mat)[1])
        {
            for(j in 1:dim(uniq.mat)[2])
            {
                if(uniq.mat[i,j]==2)
                {
                    uniq.map[i,1]=j
                    uniq.map[i,2]=j
                    break
                }
                if(uniq.mat[i,j]==1)
                {
                    if(is.na(uniq.map[i,1])==T) uniq.map[i,1]=j
                    else
                    {
                        uniq.map[i,2]=j
                        break
                    }
                }
            }
            if(is.na(uniq.map[i,1])==T)
            {
                uniq.map[i,1]=dim(uniq.mat)[2]+1
                uniq.map[i,2]=dim(uniq.mat)[2]+1
            }
            if(is.na(uniq.map[i,2])==T) uniq.map[i,2]=dim(uniq.mat)[2]+1
        }

        if (is.matrix(cov.mat)==T){cov.vec=apply(cov.mat, 1, function(x) paste(x, collapse=""))
        }else{cov.vec=cov.mat}
        cov.vec.new=unique(cbind(haplos.list$ID, cov.vec))[,-1]
        y.new=unique(cbind(haplos.list$ID, y))[,-1]
        num.E=as.vector(table(cov.vec.new))
    
        value.E=unique(cov.mat)
        if (is.matrix(value.E)==T){
            value.E=value.E[order(unique(cov.vec)),]
            len.E=nrow(value.E)
            len.dummy=ncol(value.E)
        }else{
            value.E=value.E[order(unique(cov.vec))]
            len.E=length(value.E)
            len.dummy=1
        }
    
        if (is.matrix(value.E)==T){value.E.vec=apply(value.E, 1, function(x) paste(x, collapse=""))
        }else{value.E.vec=value.E}
        index.E=match(cov.vec.new, value.E.vec)
    
        n.dep=length(names.dep)
        which.dep=rep(0, n.dep)
        which.dep=match(names.dep, names.cov)
        which.dep=sort(which.dep)
    
        which.dummy.dep=rep(0, sum(num.dummies.each.cov[which.dep]))
        len.dummy.dep=length(which.dummy.dep)
        l=0
        for (i in 1:n.dep)
        {
            step=num.dummies.each.cov[which.dep[i]]
            end.dummy.dep=sum(num.dummies.each.cov[1:which.dep[i]])
            which.dummy.dep[(l+1):(l+step)]=(end.dummy.dep+1-step):end.dummy.dep
            l=l+step
        }
    
        if (is.matrix(value.E)==T){value.dep=value.E[,which.dummy.dep]
        }else{value.dep=value.E}
        if (is.matrix(value.dep)==T){value.dep.vec=apply(value.dep, 1, function(x) paste(x, collapse=""))
        }else{value.dep.vec=value.dep}
        o=order(value.dep.vec)
        t=table(value.dep.vec)
        if (is.matrix(value.dep)==T){value.dep=unique(value.dep[o,])
        }else{value.dep=unique(value.dep[o])}
        len.dep=length(t)
        index.Etodep=rep(0, len.E)
        l=0
        for (i in 1:len.dep)
        {
            index.Etodep[o[(l+1):(l+t[i])]]=i
            l=l+t[i]
        }
    
        if (is.matrix(cov.mat)==T){dep.mat=cov.mat[,which.dummy.dep]
        }else{dep.mat=cov.mat}
        if (is.matrix(dep.mat)==T){dep.vec=apply(dep.mat, 1, function(x) paste(x, collapse=""))
        }else{dep.vec=dep.mat}
    
        j=1
        for(i in which.dep)
        {
            dep.temp0=dummy(haplos.list$nonHaploDM[,i+1])
            dep.levels0=sort(unique(haplos.list$nonHaploDM[,i+1]))
            colnames.dep.temp0=paste(colnames(haplos.list$nonHaploDM)[i+1], dep.levels0, sep="")
            if(j==1)
            {  
                dep.mat0=dep.temp0
                dep.names0=colnames.dep.temp0
            }else{
                dep.mat0=cbind(dep.mat0, dep.temp0)
                dep.names0=c(dep.names0, colnames.dep.temp0)
            }
            j=j+1
        }
    
        value.dep0=unique(dep.mat0)
        value.dep0=value.dep0[order(unique(dep.vec)),]
    
        beta=rep(start.beta, x.length+1)
        beta.out<-numeric((num.it-burn.in)*(x.length+1)) 
        lambda.out<-numeric(num.it-burn.in) 
        b.out<-numeric((num.it-burn.in)*(h.length-1)*(len.dummy.dep+1))
        freq.out<-numeric((num.it-burn.in)*len.dep*h.length)       
        D.out<-numeric(num.it-burn.in)
        
        out<-.C("mcmc_dep", as.integer(y.new), as.integer(N), as.integer(n), as.integer(num.haplo.id), as.integer(x.length), as.integer(h.length), as.integer(haplo.map), as.integer(dim(uniq.mat)[1]), as.integer(t(uniq.map)), as.integer(index.E), as.integer(t(value.E)), as.integer(num.E), as.integer(len.E), as.integer(len.dummy), as.integer(index.Etodep), as.integer(t(value.dep)), as.integer(len.dep), as.integer(len.dummy.dep), as.double(beta), as.double(lambda), as.double(freq.new), as.double(D), as.double(a), as.double(b), beta.out=as.double(beta.out), lambda.out=as.double(lambda.out), b.out=as.double(b.out), freq.out=as.double(freq.out), D.out=as.double(D.out), as.integer(num.it), as.integer(burn.in), as.double(gamma))
  
        beta.out<-matrix(out$beta.out,nrow=num.it-burn.in, byrow=TRUE)
        b.out<-matrix(out$b.out,nrow=num.it-burn.in, byrow=TRUE)
        freq.out<-matrix(out$freq.out,nrow=num.it-burn.in, byrow=TRUE)
    
        post.mean.beta<-numeric(x.length+1)
        ci.beta<-numeric((x.length+1)*2)
        post.mean.freq<-numeric(len.dep*h.length)
        ci.freq<-numeric(len.dep*h.length*2)
        ci.b<-numeric((h.length-1)*(len.dummy.dep+1)*2)
        ci.lambda<-numeric(2)
        ci.D<-numeric(2)
     
        k<-1
        for (i in 1:(x.length+1))
        {
            ci.beta[k]<-quantile(beta.out[,i], probs=0.025)
            ci.beta[k+1]<-quantile(beta.out[,i], probs=0.975)
            k<-k+2
            post.mean.beta[i]<-mean(beta.out[,i])
        }
        ci.beta<-matrix(ci.beta,nrow=x.length+1, ncol=2, byrow=TRUE)
        ci.OR<-data.frame(exp(ci.beta))
        OR<-exp(post.mean.beta)
    
        k<-1
        for (i in 1:(len.dep*h.length))
        {
            ci.freq[k]<-quantile(freq.out[,i], probs=0.025)
            ci.freq[k+1]<-quantile(freq.out[,i], probs=0.975)
            k<-k+2
            post.mean.freq[i]<-mean(freq.out[,i])
        }
        post.mean.freq<-matrix(post.mean.freq, nrow=h.length, byrow=FALSE)
        ci.freq=round(ci.freq,4)
        ci.freq<-matrix(ci.freq, ncol=2, byrow=TRUE)
        ci.freq<-apply(ci.freq, 1, function(x) paste(x, collapse=", "))
        ci.freq<-paste("(", ci.freq, sep="")
        ci.freq<-paste(ci.freq, ")", sep="")
        ci.freq<-matrix(ci.freq, nrow=h.length, byrow=FALSE)
    
        k<-1
        for (i in 1:((h.length-1)*(len.dummy.dep+1)))
        {
            ci.b[k]<-quantile(b.out[,i], probs=0.025)
            ci.b[k+1]<-quantile(b.out[,i], probs=0.975)
            k<-k+2
        }
        ci.b=round(ci.b,4)
        ci.b<-matrix(ci.b, ncol=2, byrow=TRUE)
        ci.b<-apply(ci.b, 1, function(x) paste(x, collapse=", "))
        ci.b<-paste("(", ci.b, sep="")
        ci.b<-paste(ci.b, ")", sep="")
        ci.b<-matrix(ci.b, nrow=(h.length-1), byrow=TRUE)
    
        ci.lambda<-c(quantile(out$lambda.out, probs=0.025),quantile(out$lambda.out, probs=0.975))
        ci.D<-c(quantile(out$D.out, probs=0.025),quantile(out$D.out, probs=0.975))
    
        prob.alt<-numeric(x.length+1)
        BF<-numeric(x.length+1)
        
        for (i in 1:(x.length+1))
        {
            prob.alt[i]<-length(beta.out[,i][abs(beta.out[,i]) > e])/(num.it-burn.in)
        }
    
        for (i in 1:(x.length+1))
        {
            prior.prob<-(b/(e+b))^a
            prior.odds<-prior.prob/(1-prior.prob)
            if (prob.alt[i]<=(100*prior.odds)/(100*prior.odds+1)) 
            {
                BF[i]<-round((prob.alt[i]/(1-prob.alt[i]))/prior.odds,4)
            } else
            BF[i]<-">100"
        }
    
        OR<-round(OR,4)
        ci.OR<-round(ci.OR,4)
        post.mean.freq<-round(post.mean.freq,4)
        ci.lambda<-round(ci.lambda,4)
        ci.D<-round(ci.D,4)  
    
        for(i in 1:len.dummy)
        {
            name.paste.temp<-paste(cov.names[i],haplo.names,sep=".")
            if(i==1){name.paste=name.paste.temp
            }else{name.paste=c(name.paste,name.paste.temp)}
        }
        pname<-c("beta0",haplo.names,name.paste,cov.names)
        names(BF)<-pname
        names(OR)<-pname
    
        ci.OR<-data.frame(pname,ci.OR)
        colnames(ci.OR)<-c("Name","Lower","Upper")
    
        rownames(post.mean.freq)=c(haplo.names,haplo.baseline)
        rownames(ci.freq)=c(haplo.names,haplo.baseline)
        names.step1=rep(dep.names0,len.dep)[as.logical(as.vector(t(value.dep0)))]
        names.step2=matrix(names.step1,nrow=len.dep,byrow=TRUE)
        names.step3=apply(names.step2, 1, function(x) paste(x, collapse=":"))
        colnames(post.mean.freq)=names.step3
        colnames(ci.freq)=names.step3
    
        rownames(ci.b)=haplo.names
        colnames(ci.b)=paste("gamma",0:len.dummy.dep,sep="")
    
        names(ci.lambda)<-c("Lower","Upper")
        names(ci.D)<-c("Lower","Upper")
    
        ans <- list(BF=BF, OR=OR, CI.OR=ci.OR, freq=post.mean.freq, CI.freq=ci.freq, CI.gamma=ci.b, CI.lambda=ci.lambda, CI.D=ci.D)
        }#ends here
    
        ##################################################
        #LBL-GXE
        ##################################################
        if(n.cov>=1&&interaction.model=="u"){
    
        names.cov=colnames(haplos.list$nonHaploDM)[2:(1+n.cov)]
        if (names.dep=="missing") names.dep=names.cov
        temp=cbind(haplos.list$ID, haplos.list$wt, haplos.list$nonHaploDM, haplos.list$haploDM)
        temp=temp[order(temp[,1]),]
        haplos.list$ID=temp[,1]
        haplos.list$wt=temp[,2]
        haplos.list$nonHaploDM=temp[,3:(3+n.cov)]
        haplos.list$haploDM=temp[(3+n.cov+1):ncol(temp)]
    
        if (is.null(seed)==FALSE) set.seed(seed)
        if (is.null(burn.in)==TRUE) burn.in=20000
        if (is.null(num.it)==TRUE) num.it=100000
        freq <- haplos.list$initFreq
        freq = freq[names(haplos.list$haploDM)]
        freq.names <- names(freq)
        if (haplo.baseline=="missing") haplo.baseline <- freq.names[which.max(freq)]
        column.subset <- colnames(haplos.list$haploDM) != haplo.baseline
        haplo.mat=haplos.list$haploDM[,column.subset]
        haplo.names=colnames(haplo.mat)
    
        freq.new<-freq[column.subset]
        freq.new[length(freq.new)+1]<-freq[haplo.baseline]
       
        if (cov.baseline=="missing") cov.baseline = rep(0, n.cov)
        haplos.list$nonHaploDM=as.matrix(haplos.list$nonHaploDM)
        num.dummies.each.cov=rep(0, n.cov)
        for(i in 1:n.cov)
        {
            cov.temp=dummy(haplos.list$nonHaploDM[,i+1])
            cov.levels=sort(unique(haplos.list$nonHaploDM[,i+1]))
            cov.subset=(cov.levels!=cov.baseline[i])
            cov.temp=cov.temp[,cov.subset]
            colnames.cov.temp=paste(colnames(haplos.list$nonHaploDM)[i+1], cov.levels[cov.subset], sep="")
            num.dummies.each.cov[i]=length(cov.levels)-1
            if(i==1)
            {
                cov.mat=cov.temp
                cov.names=colnames.cov.temp
            }else{
                cov.mat=cbind(cov.mat, cov.temp)
                cov.names=c(cov.names, colnames.cov.temp)
            }
        }
    
        y <- haplos.list$nonHaploDM[,1]
        N <- sum(haplos.list$wt)
        n=dim(haplo.mat)[1]
        num.haplo.id=as.vector(table(haplos.list$ID))
    
        h.length=ncol(haplo.mat)+1
        if (is.matrix(cov.mat)==T){x.length=ncol(haplo.mat)+ncol(cov.mat)+ncol(haplo.mat)*ncol(cov.mat)
        }else{x.length=ncol(haplo.mat)+1+ncol(haplo.mat)}
    
        haplo.map<-matrix(rep(NA,2*(dim(haplo.mat)[1])),ncol=2)
        for(i in 1:dim(haplo.mat)[1])
        {
            for(j in 1:dim(haplo.mat)[2])
            {
                if(haplo.mat[i,j]==2)
                {
                    haplo.map[i,1]=j
                    haplo.map[i,2]=j
                    break
                }
                if(haplo.mat[i,j]==1)
                {
                    if(is.na(haplo.map[i,1])==T) haplo.map[i,1]=j
                    else
                    {
                        haplo.map[i,2]=j
                        break
                    }
                }
            }
            if(is.na(haplo.map[i,1])==T)
            {
                haplo.map[i,1]=dim(haplo.mat)[2]+1
                haplo.map[i,2]=dim(haplo.mat)[2]+1
            }
            if(is.na(haplo.map[i,2])==T) haplo.map[i,2]=dim(haplo.mat)[2]+1
        }  
    
        uniq.mat<-unique(haplo.mat)
        uniq.map<-matrix(rep(NA,2*(dim(uniq.mat)[1])),ncol=2)
        for(i in 1:dim(uniq.mat)[1])
        {
            for(j in 1:dim(uniq.mat)[2])
            {
                if(uniq.mat[i,j]==2)
                {
                    uniq.map[i,1]=j
                    uniq.map[i,2]=j
                    break
                }
                if(uniq.mat[i,j]==1)
                {
                    if(is.na(uniq.map[i,1])==T) uniq.map[i,1]=j
                    else
                    {
                        uniq.map[i,2]=j
                        break
                    }
                }
            }
            if(is.na(uniq.map[i,1])==T)
            {
                uniq.map[i,1]=dim(uniq.mat)[2]+1
                uniq.map[i,2]=dim(uniq.mat)[2]+1
            }
            if(is.na(uniq.map[i,2])==T) uniq.map[i,2]=dim(uniq.mat)[2]+1
        }
    
        if (is.matrix(cov.mat)==T){cov.vec=apply(cov.mat, 1, function(x) paste(x, collapse=""))
        }else{cov.vec=cov.mat}
        cov.vec.new=unique(cbind(haplos.list$ID, cov.vec))[,-1]
        y.new=unique(cbind(haplos.list$ID, y))[,-1]
        num.E=as.vector(table(cov.vec.new))
    
        value.E=unique(cov.mat)
        if (is.matrix(value.E)==T){
            value.E=value.E[order(unique(cov.vec)),]
            len.E=nrow(value.E)
            len.dummy=ncol(value.E)
        }else{
            value.E=value.E[order(unique(cov.vec))]
            len.E=length(value.E)
            len.dummy=1
        }
    
        if (is.matrix(value.E)==T){value.E.vec=apply(value.E, 1, function(x) paste(x, collapse=""))
        }else{value.E.vec=value.E}
        index.E=match(cov.vec.new, value.E.vec)
    
        n.dep=length(names.dep)
        which.dep=rep(0, n.dep)
        which.dep=match(names.dep, names.cov)
        which.dep=sort(which.dep)
    
        which.dummy.dep=rep(0, sum(num.dummies.each.cov[which.dep]))
        len.dummy.dep=length(which.dummy.dep)
        l=0
        for (i in 1:n.dep)
        {
            step=num.dummies.each.cov[which.dep[i]]
            end.dummy.dep=sum(num.dummies.each.cov[1:which.dep[i]])
            which.dummy.dep[(l+1):(l+step)]=(end.dummy.dep+1-step):end.dummy.dep
            l=l+step
        }
    
        if (is.matrix(value.E)==T){value.dep=value.E[,which.dummy.dep]
        }else{value.dep=value.E}
        if (is.matrix(value.dep)==T){value.dep.vec=apply(value.dep, 1, function(x) paste(x, collapse=""))
        }else{value.dep.vec=value.dep}
        o=order(value.dep.vec)
        t=table(value.dep.vec)
        if (is.matrix(value.dep)==T){value.dep=unique(value.dep[o,])
        }else{value.dep=unique(value.dep[o])}
        len.dep=length(t)
        index.Etodep=rep(0, len.E)
        l=0
        for (i in 1:len.dep)
        {
            index.Etodep[o[(l+1):(l+t[i])]]=i
            l=l+t[i]
        }
    
        if (is.matrix(cov.mat)==T){dep.mat=cov.mat[,which.dummy.dep]
        }else{dep.mat=cov.mat}
        if (is.matrix(dep.mat)==T){dep.vec=apply(dep.mat, 1, function(x) paste(x, collapse=""))
        }else{dep.vec=dep.mat}
    
        j=1
        for(i in which.dep)
        {
            dep.temp0=dummy(haplos.list$nonHaploDM[,i+1])
            dep.levels0=sort(unique(haplos.list$nonHaploDM[,i+1]))
            colnames.dep.temp0=paste(colnames(haplos.list$nonHaploDM)[i+1], dep.levels0, sep="")
            if(j==1)
            {  
                dep.mat0=dep.temp0
                dep.names0=colnames.dep.temp0
            }else{
                dep.mat0=cbind(dep.mat0, dep.temp0)
                dep.names0=c(dep.names0, colnames.dep.temp0)
            }
            j=j+1
        }
    
        value.dep0=unique(dep.mat0)
        value.dep0=value.dep0[order(unique(dep.vec)),]
    
        beta=rep(start.beta, x.length+1)
        beta.out<-numeric((num.it-burn.in)*(x.length+1))
        lambda.out<-numeric(num.it-burn.in) 
        freq.out<-numeric((num.it-burn.in)*h.length)
        freq.out.indep<-numeric((num.it-burn.in)*h.length)
        freq.out.dep<-numeric((num.it-burn.in)*len.dep*h.length)
        b.out.dep<-numeric((num.it-burn.in)*(h.length-1)*(len.dummy.dep+1))
        D.out<-numeric(num.it-burn.in) 
        count.model.1<-0
        count.model.2<-0
    
        out<-.C("mcmc", as.integer(y.new), as.integer(N), as.integer(n), as.integer(num.haplo.id), as.integer(x.length), as.integer(h.length), as.integer(haplo.map), as.integer(dim(uniq.mat)[1]), as.integer(t(uniq.map)), as.integer(index.E), as.integer(t(value.E)), as.integer(num.E), as.integer(len.E), as.integer(len.dummy), as.integer(index.Etodep), as.integer(t(value.dep)), as.integer(len.dep), as.integer(len.dummy.dep), as.double(beta), as.double(lambda), as.double(freq.new), as.double(D), as.double(a), as.double(b), beta.out=as.double(beta.out), lambda.out=as.double(lambda.out), freq.out=as.double(freq.out), freq.out.indep=as.double(freq.out.indep), freq.out.dep=as.double(freq.out.dep), b.out.dep=as.double(b.out.dep), D.out=as.double(D.out), as.integer(num.it), as.integer(burn.in), count.model.1=as.integer(count.model.1), count.model.2=as.integer(count.model.2))
  
        beta.out<-matrix(out$beta.out,nrow=num.it-burn.in, byrow=TRUE)
        freq.out<-matrix(out$freq.out,nrow=num.it-burn.in, byrow=TRUE)
        freq.out.indep<-matrix(out$freq.out.indep,nrow=num.it-burn.in, byrow=TRUE)
        freq.out.dep<-matrix(out$freq.out.dep,nrow=num.it-burn.in, byrow=TRUE)
        b.out.dep<-matrix(out$b.out.dep,nrow=num.it-burn.in, byrow=TRUE)
    
        post.mean.beta<-numeric(x.length+1)
        ci.beta<-numeric((x.length+1)*2)
        post.mean.freq<-numeric(h.length)
        post.mean.freq.indep<-numeric(h.length)
        post.mean.freq.dep<-numeric(len.dep*h.length)
        ci.freq<-numeric(h.length*2)
        ci.freq.indep<-numeric(h.length*2)
        ci.freq.dep<-numeric(len.dep*h.length*2)
        ci.b.dep<-numeric((h.length-1)*(len.dummy.dep+1)*2)
        ci.lambda<-numeric(2)
        ci.D<-numeric(2)
     
        k<-1
        for (i in 1:(x.length+1))
        {
            ci.beta[k]<-quantile(beta.out[,i], probs=0.025)
            ci.beta[k+1]<-quantile(beta.out[,i], probs=0.975)
            k<-k+2
            post.mean.beta[i]<-mean(beta.out[,i])
        }
        ci.beta<-matrix(ci.beta,nrow=x.length+1, ncol=2, byrow=TRUE)
        ci.OR<-data.frame(exp(ci.beta))
        OR<-exp(post.mean.beta)
        
        k<-1
        sub=apply(freq.out.indep, 1, function(x) any(x!=0))
        freq.out.indep=freq.out.indep[sub,]
        for (i in 1:(h.length))
        {
            ci.freq[k]<-quantile(freq.out[,i], probs=0.025)
            ci.freq[k+1]<-quantile(freq.out[,i], probs=0.975)
            ci.freq.indep[k]<-quantile(freq.out.indep[,i], probs=0.025)
            ci.freq.indep[k+1]<-quantile(freq.out.indep[,i], probs=0.975)
            k<-k+2
            post.mean.freq[i]<-mean(freq.out[,i])
            post.mean.freq.indep[i]<-mean(freq.out.indep[,i])
        } 
        k<-1
        sub=apply(freq.out.dep, 1, function(x) any(x!=0))
        freq.out.dep=freq.out.dep[sub,]
        for (i in 1:(len.dep*h.length))
        {
            ci.freq.dep[k]<-quantile(freq.out.dep[,i], probs=0.025)
            ci.freq.dep[k+1]<-quantile(freq.out.dep[,i], probs=0.975)
            k<-k+2
            post.mean.freq.dep[i]<-mean(freq.out.dep[,i])
        }
        post.mean.freq.dep<-matrix(post.mean.freq.dep, nrow=h.length, byrow=FALSE)
        ci.freq.dep=round(ci.freq.dep,4)
        ci.freq.dep<-matrix(ci.freq.dep, ncol=2, byrow=TRUE)
        ci.freq.dep<-apply(ci.freq.dep, 1, function(x) paste(x, collapse=", "))
        ci.freq.dep<-paste("(", ci.freq.dep, sep="")
        ci.freq.dep<-paste(ci.freq.dep, ")", sep="")
        ci.freq.dep<-matrix(ci.freq.dep, nrow=h.length, byrow=FALSE)
    
        k<-1
        sub=apply(b.out.dep, 1, function(x) any(x!=0))
        b.out.dep=b.out.dep[sub,]
        for (i in 1:((h.length-1)*(len.dummy.dep+1)))
        {
            ci.b.dep[k]<-quantile(b.out.dep[,i], probs=0.025)
            ci.b.dep[k+1]<-quantile(b.out.dep[,i], probs=0.975)
            k<-k+2
        }
        ci.b.dep=round(ci.b.dep,4)
        ci.b.dep<-matrix(ci.b.dep, ncol=2, byrow=TRUE)
        ci.b.dep<-apply(ci.b.dep, 1, function(x) paste(x, collapse=", "))
        ci.b.dep<-paste("(", ci.b.dep, sep="")
        ci.b.dep<-paste(ci.b.dep, ")", sep="")
        ci.b.dep<-matrix(ci.b.dep, nrow=(h.length-1), byrow=TRUE)
    
        ci.lambda<-c(quantile(out$lambda.out, probs=0.025),quantile(out$lambda.out, probs=0.975))
        ci.D<-c(quantile(out$D.out, probs=0.025),quantile(out$D.out, probs=0.975))
    
        prob.alt<-numeric(x.length+1)
        BF<-numeric(x.length+1)
        
        for (i in 1:(x.length+1))
        {
            prob.alt[i]<-length(beta.out[,i][abs(beta.out[,i]) > e])/(num.it-burn.in)
        }
    
        for (i in 1:(x.length+1))
        {
            prior.prob<-(b/(e+b))^a
            prior.odds<-prior.prob/(1-prior.prob)
            if (prob.alt[i]<=(100*prior.odds)/(100*prior.odds+1)) 
            {
                BF[i]<-round((prob.alt[i]/(1-prob.alt[i]))/prior.odds,4)
            } else
            BF[i]<-">100"
        }
    
        OR<-round(OR,4)
        ci.OR<-round(ci.OR,4)
        post.mean.freq<-round(post.mean.freq,4)
        post.mean.freq.indep<-round(post.mean.freq.indep,4)
        post.mean.freq.dep<-round(post.mean.freq.dep,4)
        ci.freq<-round(ci.freq,4)
        ci.freq.indep<-round(ci.freq.indep,4)
        ci.lambda<-round(ci.lambda,4)
        ci.D<-round(ci.D,4) 
    
        for(i in 1:len.dummy)
        {
            name.paste.temp<-paste(cov.names[i],haplo.names,sep=".")
            if(i==1){name.paste=name.paste.temp
            }else{name.paste=c(name.paste,name.paste.temp)}
        }
        pname<-c("beta0",haplo.names,name.paste,cov.names)
        names(BF)<-pname
        names(OR)<-pname
    
        ci.OR<-data.frame(pname,ci.OR)
        colnames(ci.OR)<-c("Name","Lower","Upper")
    
        ci.freq<-matrix(ci.freq,nrow=(h.length),ncol=2,byrow=TRUE)
        ci.freq.indep<-matrix(ci.freq.indep,nrow=(h.length),ncol=2,byrow=TRUE)
        ci.freq<-data.frame(c(haplo.names,haplo.baseline),ci.freq)
        ci.freq.indep<-data.frame(c(haplo.names,haplo.baseline),ci.freq.indep)
        colnames(ci.freq)<-c("Name","Lower","Upper")
        colnames(ci.freq.indep)<-c("Name","Lower","Upper")
        names(post.mean.freq)<-c(haplo.names,haplo.baseline)
        names(post.mean.freq.indep)<-c(haplo.names,haplo.baseline)
    
        rownames(post.mean.freq.dep)=c(haplo.names,haplo.baseline)
        rownames(ci.freq.dep)=c(haplo.names,haplo.baseline)
        names.step1=rep(dep.names0,len.dep)[as.logical(as.vector(t(value.dep0)))]
        names.step2=matrix(names.step1,nrow=len.dep,byrow=TRUE)
        names.step3=apply(names.step2, 1, function(x) paste(x, collapse=":"))
        colnames(post.mean.freq.dep)=names.step3
        colnames(ci.freq.dep)=names.step3
    
        rownames(ci.b.dep)=haplo.names
        colnames(ci.b.dep)=paste("gamma",0:len.dummy.dep,sep="")
    
        names(ci.lambda)<-c("Lower","Upper")
        names(ci.D)<-c("Lower","Upper")
    
        percentage.indep=out$count.model.1/(num.it-burn.in)
        percentage.dep=out$count.model.2/(num.it-burn.in)
    
        ans <- list(BF=BF, OR=OR, CI.OR=ci.OR, freq=post.mean.freq, CI.freq=ci.freq, CI.gamma=ci.b.dep, CI.lambda=ci.lambda, CI.D=ci.D, percentage.indep=percentage.indep, percentage.dep=percentage.dep)
        }#ends here
    }

    ####################################################################################################
    #Complex sampling methods
    ####################################################################################################
    if(complex.sampling==TRUE)
    {
        if (is.null(n.stra)==TRUE) print("Warning: n.stra is missing!")

        weight=haplos.list$nonHaploDM[,2]
        haplos.list$nonHaploDM=haplos.list$nonHaploDM[,-2]
        n.var=dim(haplos.list$nonHaploDM)[2]-1
        names.var=colnames(haplos.list$nonHaploDM)[2:(1+n.var)]

        if (n.var==n.stra) interaction.cov=F

        if (names.dep=="missing") names.dep=names.var
        temp=cbind(haplos.list$ID, haplos.list$wt, weight, haplos.list$nonHaploDM, haplos.list$haploDM)
        temp=temp[order(temp[,1]),]
        haplos.list$ID=temp[,1]
        haplos.list$wt=temp[,2]
        weight=temp[,3]
        haplos.list$nonHaploDM=temp[,4:(4+n.var)]
        haplos.list$haploDM=temp[(4+n.var+1):ncol(temp)]

        if (is.null(seed)==FALSE) set.seed(seed)
        if (is.null(burn.in)==TRUE) burn.in=20000
        if (is.null(num.it)==TRUE) num.it=120000
        freq <- haplos.list$initFreq
        freq = freq[names(haplos.list$haploDM)]
        freq.names <- names(freq)
        if (haplo.baseline=="missing") haplo.baseline <- freq.names[which.max(freq)]
        column.subset <- colnames(haplos.list$haploDM) != haplo.baseline
        haplo.mat=haplos.list$haploDM[,column.subset]
        haplo.names=colnames(haplo.mat)
    
        freq.new<-freq[column.subset]
        freq.new[length(freq.new)+1]<-freq[haplo.baseline]
        
        if(var.baseline=="missing") var.baseline = rep(0, n.var)
        haplos.list$nonHaploDM=as.matrix(haplos.list$nonHaploDM)
        num.dummies.each.var=rep(0, n.var)
        for(i in 1:n.var)
        {
            var.temp=dummy(haplos.list$nonHaploDM[,i+1])
            var.levels=sort(unique(haplos.list$nonHaploDM[,i+1]))
            var.subset=(var.levels!=var.baseline[i])
            var.temp=var.temp[,var.subset]
            colnames.var.temp=paste(colnames(haplos.list$nonHaploDM)[i+1], var.levels[var.subset], sep=".")
            num.dummies.each.var[i]=length(var.levels)-1
            if(i==1)
            {
                var.mat=var.temp
                var.names=colnames.var.temp
            }else{
                var.mat=cbind(var.mat, var.temp)
                var.names=c(var.names, colnames.var.temp)
            }
        }

        y <- haplos.list$nonHaploDM[,1]
        N <- sum(haplos.list$wt)
        n=dim(haplo.mat)[1]
        num.haplo.id=as.vector(table(haplos.list$ID))
    
        h.length=ncol(haplo.mat)+1
        len.dummy.stra=sum(num.dummies.each.var[1:n.stra])
        len.dummy.cov=sum(num.dummies.each.var[-c(1:n.stra)])
        if (is.matrix(var.mat)==T){
            if (interaction.stra==T & interaction.cov==T){
                x.length=ncol(haplo.mat)+ncol(var.mat)+ncol(haplo.mat)*ncol(var.mat)
            } else if (interaction.stra==T & interaction.cov==F){
                x.length=ncol(haplo.mat)+ncol(var.mat)+ncol(haplo.mat)*len.dummy.stra
            } else if (interaction.stra==F & interaction.cov==T){
                x.length=ncol(haplo.mat)+ncol(var.mat)+ncol(haplo.mat)*len.dummy.cov
            } else {x.length=ncol(haplo.mat)+ncol(var.mat)}
        }else{
            if (interaction.stra==T){
                x.length=ncol(haplo.mat)+1+ncol(haplo.mat)
            } else {
            x.length=ncol(haplo.mat)+1}}
    
        haplo.map<-matrix(rep(NA,2*(dim(haplo.mat)[1])),ncol=2)
        for(i in 1:dim(haplo.mat)[1])
        {
            for(j in 1:dim(haplo.mat)[2])
            {
                if(haplo.mat[i,j]==2)
                {
                    haplo.map[i,1]=j
                    haplo.map[i,2]=j
                    break
                }
                if(haplo.mat[i,j]==1)
                {
                    if(is.na(haplo.map[i,1])==T) haplo.map[i,1]=j
                    else
                    {
                        haplo.map[i,2]=j
                        break
                    }
                }
            }
            if(is.na(haplo.map[i,1])==T)
            {
                haplo.map[i,1]=dim(haplo.mat)[2]+1
                haplo.map[i,2]=dim(haplo.mat)[2]+1
            }
            if(is.na(haplo.map[i,2])==T) haplo.map[i,2]=dim(haplo.mat)[2]+1
        }  

        uniq.mat<-unique(haplo.mat)
        uniq.map<-matrix(rep(NA,2*(dim(uniq.mat)[1])),ncol=2)
        for(i in 1:dim(uniq.mat)[1])
        {
            for(j in 1:dim(uniq.mat)[2])
            {
                if(uniq.mat[i,j]==2)
                {
                    uniq.map[i,1]=j
                    uniq.map[i,2]=j
                    break
                }
                if(uniq.mat[i,j]==1)
                {
                    if(is.na(uniq.map[i,1])==T) uniq.map[i,1]=j
                    else
                    {
                        uniq.map[i,2]=j
                        break
                    }
                }
            }
            if(is.na(uniq.map[i,1])==T)
            {
                uniq.map[i,1]=dim(uniq.mat)[2]+1
                uniq.map[i,2]=dim(uniq.mat)[2]+1
            }
            if(is.na(uniq.map[i,2])==T) uniq.map[i,2]=dim(uniq.mat)[2]+1
	}
    
        if (is.matrix(var.mat)==T){var.vec=apply(var.mat, 1, function(x) paste(x, collapse=""))
        }else{var.vec=var.mat}
        var.vec.new=unique(cbind(haplos.list$ID, var.vec))[,-1]
        y.new=unique(cbind(haplos.list$ID, y))[,-1]
        weight=unique(cbind(haplos.list$ID, weight))[,-1]
        weight0=weight[order(var.vec.new)]
        num.E=as.vector(table(var.vec.new))
        num.E.wt=rep(0,length(num.E))
        l=0
        for (i in 1:length(num.E.wt))
        {
            num.E.wt[i]=sum(weight0[(l+1):(l+num.E[i])])
            l=l+num.E[i]
        }
    
        value.E=unique(var.mat)
        if (is.matrix(value.E)==T){
            value.E=value.E[order(unique(var.vec)),]
            len.E=nrow(value.E)
            len.dummy=ncol(value.E)
        }else{
            value.E=value.E[order(unique(var.vec))]
            len.E=length(value.E)
            len.dummy=1
        }
    
        if (is.matrix(value.E)==T){value.E.vec=apply(value.E, 1, function(x) paste(x, collapse=""))
        }else{value.E.vec=value.E}
        index.E=match(var.vec.new, value.E.vec)

        n.dep=length(names.dep)
        which.dep=rep(0, n.dep)
        which.dep=match(names.dep, names.var)
        which.dep=sort(which.dep)
    
        which.dummy.dep=rep(0, sum(num.dummies.each.var[which.dep]))
        len.dummy.dep=length(which.dummy.dep)
        l=0
        for (i in 1:n.dep)
        {
            step=num.dummies.each.var[which.dep[i]]
            end.dummy.dep=sum(num.dummies.each.var[1:which.dep[i]])
            which.dummy.dep[(l+1):(l+step)]=(end.dummy.dep+1-step):end.dummy.dep
            l=l+step
        }
    
        if (is.matrix(value.E)==T){value.dep=value.E[,which.dummy.dep]
        }else{value.dep=value.E}
        if (is.matrix(value.dep)==T){value.dep.vec=apply(value.dep, 1, function(x) paste(x, collapse=""))
        }else{value.dep.vec=value.dep}
        o=order(value.dep.vec)
        t=table(value.dep.vec)
        if (is.matrix(value.dep)==T){value.dep=unique(value.dep[o,])
        }else{value.dep=unique(value.dep[o])}
        len.dep=length(t)
        index.Etodep=rep(0, len.E)
        l=0
        for (i in 1:len.dep)
        {
            index.Etodep[o[(l+1):(l+t[i])]]=i
            l=l+t[i]
        }
    
        if (is.matrix(var.mat)==T){dep.mat=var.mat[,which.dummy.dep]
        }else{dep.mat=var.mat}
        if (is.matrix(dep.mat)==T){dep.vec=apply(dep.mat, 1, function(x) paste(x, collapse=""))
        }else{dep.vec=dep.mat}
    
        j=1
        for(i in which.dep)
        {
            dep.temp0=dummy(haplos.list$nonHaploDM[,i+1])
            dep.levels0=sort(unique(haplos.list$nonHaploDM[,i+1]))
            colnames.dep.temp0=paste(colnames(haplos.list$nonHaploDM)[i+1], dep.levels0, sep=".")
            if(j==1)
            {  
                dep.mat0=dep.temp0
                dep.names0=colnames.dep.temp0
            }else{
                dep.mat0=cbind(dep.mat0, dep.temp0)
                dep.names0=c(dep.names0, colnames.dep.temp0)
            }
            j=j+1
        }
    
        value.dep0=unique(dep.mat0)
        value.dep0=value.dep0[order(unique(dep.vec)),]
    
        beta=rep(start.beta, x.length+1)
        beta.out<-numeric((num.it-burn.in)*(x.length+1)) 
        lambda.out<-numeric(num.it-burn.in) 
        b.out<-numeric((num.it-burn.in)*(h.length-1)*(len.dummy.dep+1))
        freq.out<-numeric((num.it-burn.in)*len.dep*h.length)       
        D.out<-numeric(num.it-burn.in)

        out<-.C("mcmc_complex", as.integer(y.new), as.double(weight), as.integer(N), as.integer(n), as.integer(num.haplo.id), as.integer(x.length), as.integer(h.length), as.integer(haplo.map), as.integer(dim(uniq.mat)[1]), as.integer(t(uniq.map)), as.integer(index.E), as.integer(t(value.E)), as.integer(num.E.wt), as.integer(len.E), as.integer(len.dummy), as.integer(index.Etodep), as.integer(t(value.dep)), as.integer(len.dep), as.integer(len.dummy.dep), as.double(beta), as.double(lambda), as.double(freq.new), as.double(D), as.double(a), as.double(b), beta.out=as.double(beta.out), lambda.out=as.double(lambda.out), b.out=as.double(b.out), freq.out=as.double(freq.out), D.out=as.double(D.out), as.integer(num.it), as.integer(burn.in), as.integer(interaction.stra), as.integer(interaction.cov), as.integer(len.dummy.stra), as.integer(len.dummy.cov))
  
        beta.out<-matrix(out$beta.out,nrow=num.it-burn.in, byrow=TRUE)
        b.out<-matrix(out$b.out,nrow=num.it-burn.in, byrow=TRUE)
        freq.out<-matrix(out$freq.out,nrow=num.it-burn.in, byrow=TRUE)
    
        ci.beta<-numeric((x.length+1)*2)
        post.mean.beta<-numeric(x.length+1)
        ci.beta<-numeric((x.length+1)*2)
        post.mean.freq<-numeric(len.dep*h.length)
        ci.freq<-numeric(len.dep*h.length*2)
        ci.b<-numeric((h.length-1)*(len.dummy.dep+1)*2)
        ci.lambda<-numeric(2)
        ci.D<-numeric(2)

        k<-1
        for (i in 1:(x.length+1))
        {
            ci.beta[k]<-quantile(beta.out[,i], probs=0.025)
            ci.beta[k+1]<-quantile(beta.out[,i], probs=0.975)
            k<-k+2
            post.mean.beta[i]<-mean(beta.out[,i])
        }
        ci.beta<-matrix(ci.beta,nrow=x.length+1, ncol=2, byrow=TRUE)
        ci.OR<-data.frame(exp(ci.beta))
        OR<-exp(post.mean.beta)
    
        k<-1
        for (i in 1:(len.dep*h.length))
        {
            ci.freq[k]<-quantile(freq.out[,i], probs=0.025)
            ci.freq[k+1]<-quantile(freq.out[,i], probs=0.975)
            k<-k+2
            post.mean.freq[i]<-mean(freq.out[,i])
        }
        post.mean.freq<-matrix(post.mean.freq, nrow=h.length, byrow=FALSE)
        ci.freq=round(ci.freq,4)
        ci.freq<-matrix(ci.freq, ncol=2, byrow=TRUE)
        ci.freq<-apply(ci.freq, 1, function(x) paste(x, collapse=", "))
        ci.freq<-paste("(", ci.freq, sep="")
        ci.freq<-paste(ci.freq, ")", sep="")
        ci.freq<-matrix(ci.freq, nrow=h.length, byrow=FALSE)
    
        k<-1
        for (i in 1:((h.length-1)*(len.dummy.dep+1)))
        {
            ci.b[k]<-quantile(b.out[,i], probs=0.025)
            ci.b[k+1]<-quantile(b.out[,i], probs=0.975)
            k<-k+2
        }
        ci.b=round(ci.b,4)
        ci.b<-matrix(ci.b, ncol=2, byrow=TRUE)
        ci.b<-apply(ci.b, 1, function(x) paste(x, collapse=", "))
        ci.b<-paste("(", ci.b, sep="")
        ci.b<-paste(ci.b, ")", sep="")
        ci.b<-matrix(ci.b, nrow=(h.length-1), byrow=TRUE)
    
        ci.lambda<-c(quantile(out$lambda.out, probs=0.025),quantile(out$lambda.out, probs=0.975))
        ci.D<-c(quantile(out$D.out, probs=0.025),quantile(out$D.out, probs=0.975))
    
        prob.alt<-numeric(x.length+1)
        BF<-numeric(x.length+1)
        
        for (i in 1:(x.length+1))
        {
            prob.alt[i]<-length(beta.out[,i][abs(beta.out[,i]) > e])/(num.it-burn.in)
        }
    
        for (i in 1:(x.length+1))
        {
            prior.prob<-(b/(e+b))^a
            prior.odds<-prior.prob/(1-prior.prob)
            if (prob.alt[i]<=(100*prior.odds)/(100*prior.odds+1)) 
            {
                BF[i]<-round((prob.alt[i]/(1-prob.alt[i]))/prior.odds,4)
            } else
            BF[i]<-">100"
        }
    
        OR<-round(OR,4)
        ci.OR<-round(ci.OR,4)
        post.mean.freq<-round(post.mean.freq,4)
        ci.lambda<-round(ci.lambda,4)
        ci.D<-round(ci.D,4)  

        if (interaction.stra==T & interaction.cov==T){
            for(i in 1:len.dummy)
            {
                name.paste.temp<-paste(var.names[i],haplo.names,sep=" x ")
                if(i==1){name.paste=name.paste.temp
                }else{name.paste=c(name.paste,name.paste.temp)}
            }
            pname<-c("beta0",haplo.names,name.paste,var.names)
        } else if (interaction.stra==T & interaction.cov==F){
            for(i in 1:len.dummy.stra)
            {
                name.paste.temp<-paste(var.names[i],haplo.names,sep=" x ")
                if(i==1){name.paste=name.paste.temp
                }else{name.paste=c(name.paste,name.paste.temp)}
            }
            pname<-c("beta0",haplo.names,name.paste,var.names)
        } else if (interaction.stra==F & interaction.cov==T){
            for(i in 1:len.dummy.cov)
            {
                name.paste.temp<-paste(var.names[i+len.dummy.stra],haplo.names,sep=" x ")
                if(i==1){name.paste=name.paste.temp
                }else{name.paste=c(name.paste,name.paste.temp)}
            }
            pname<-c("beta0",haplo.names,name.paste,var.names)
        } else {pname<-c("beta0",haplo.names,var.names)}
        names(BF)<-pname
        names(OR)<-pname

        ci.OR<-data.frame(pname,ci.OR)
        colnames(ci.OR)<-c("Name","Lower","Upper")
    
        rownames(post.mean.freq)=c(haplo.names,haplo.baseline)
        rownames(ci.freq)=c(haplo.names,haplo.baseline)
        names.step1=rep(dep.names0,len.dep)[as.logical(as.vector(t(value.dep0)))]
        names.step2=matrix(names.step1,nrow=len.dep,byrow=TRUE)
        names.step3=apply(names.step2, 1, function(x) paste(x, collapse=":"))
        colnames(post.mean.freq)=names.step3
        colnames(ci.freq)=names.step3
    
        rownames(ci.b)=haplo.names
        colnames(ci.b)=paste("gamma",0:len.dummy.dep,sep="")
    
        names(ci.lambda)<-c("Lower","Upper")
        names(ci.D)<-c("Lower","Upper")
    
        ans <- list(BF=BF, OR=OR, CI.OR=ci.OR, freq=post.mean.freq, CI.freq=ci.freq, CI.gamma=ci.b, CI.lambda=ci.lambda, CI.D=ci.D)
    }#ends here

    return (ans)
}

